@@ -571,6 +571,13 @@ mastermidibus::mastermidibus()
     m_num_out_buses = 0;
     m_num_in_buses = 0;
 
+	for( int i=0; i<c_maxBuses; ++i ){
+		m_buses_in_active[i] = false;	
+		m_buses_out_active[i] = false;
+		m_buses_in_init[i] = false;	
+		m_buses_out_init[i] = false;
+	}
+
     /* open the sequencer client */
     ret = snd_seq_open(&m_alsa_seq, "default",  SND_SEQ_OPEN_DUPLEX, 0);
 
@@ -631,7 +638,8 @@ mastermidibus::mastermidibus()
 		    
 		    m_buses_out[m_num_out_buses]->init_out();
 		    m_buses_out_active[m_num_out_buses] = true;		
-	
+			m_buses_out_init[m_num_out_buses] = true;		
+
 		    m_num_out_buses++;
 		}	
 		
@@ -650,7 +658,7 @@ mastermidibus::mastermidibus()
 		    	
 			m_buses_in[m_num_in_buses]->init_in();
 			m_buses_in_active[m_num_in_buses] = true;	
-
+			m_buses_in_init[m_num_in_buses] = true;	
 		    m_num_in_buses++;
 		}	
 	    }
@@ -734,7 +742,7 @@ void
 mastermidibus::play( unsigned char a_bus, event *a_e24, unsigned char a_channel )
 {
 	lock();
-	if ( m_buses_out_active[a_bus] ){
+	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
 		m_buses_out[a_bus]->play( a_e24, a_channel );
 	}
 	unlock();
@@ -744,7 +752,7 @@ mastermidibus::play( unsigned char a_bus, event *a_e24, unsigned char a_channel
 void 
 mastermidibus::set_clock( unsigned char a_bus, bool a_clocking )
 {
-	if ( m_buses_out_active[a_bus] ){
+	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
 		m_buses_out[a_bus]->set_clock( a_clocking );
 	}
 }
@@ -752,7 +760,7 @@ mastermidibus::set_clock( unsigned char a_bus, bool a_clocking )
 bool 
 mastermidibus::get_clock( unsigned char a_bus )
 {
-	if ( m_buses_out_active[a_bus] ){
+	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
 		return m_buses_out[a_bus]->get_clock();
 	}
 	return false;
@@ -762,19 +770,25 @@ mastermidibus::get_clock( unsigned char a_bus )
 string
 mastermidibus::get_midi_bus_name( int a_bus )
 {
-	if ( m_buses_out_active[a_bus] ){
+	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
 		return m_buses_out[a_bus]->get_name();
 	}
 	
 	/* copy names */
-    char tmp[60];
-    snprintf( tmp, 59, "[%d] %d:%d (disconnected)",
-			  a_bus,
-			  m_buses_out[a_bus]->get_client(),
-			  m_buses_out[a_bus]->get_port() );
+	char tmp[60];
 
-    string ret = tmp;
-	return ret;
+	if ( m_buses_out_init[a_bus] ){
+		snprintf( tmp, 59, "[%d] %d:%d (disconnected)",
+				  a_bus,
+				  m_buses_out[a_bus]->get_client(),
+				  m_buses_out[a_bus]->get_port() );
+	} else {
+		snprintf( tmp, 59, "[%d] (unconnected)",
+				  a_bus );
+	}
+
+	string ret = tmp;
+	return ret;	
 	
 }
 
@@ -860,30 +874,33 @@ mastermidibus::port_start( int a_client, int a_port )
 	if ( snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
 
 		/* the outs */
-		if ( (cap & SND_SEQ_PORT_CAP_SUBS_WRITE) != 0 &&
-		     snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
+		if ( (cap & (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE ))
+			 == (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE )
+			 && snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
 		    
 		    m_buses_out[bus_slot] = 
-			new midibus( snd_seq_client_id( m_alsa_seq ),
-				     snd_seq_port_info_get_client(pinfo),
-				     snd_seq_port_info_get_port(pinfo),
-				     m_alsa_seq,
-				     snd_seq_client_info_get_name(cinfo),
-				     snd_seq_port_info_get_name(pinfo),
-				     m_num_out_buses, m_queue );
+				new midibus( snd_seq_client_id( m_alsa_seq ),
+							 snd_seq_port_info_get_client(pinfo),
+							 snd_seq_port_info_get_port(pinfo),
+							 m_alsa_seq,
+							 snd_seq_client_info_get_name(cinfo),
+							 snd_seq_port_info_get_name(pinfo),
+							 m_num_out_buses, m_queue );
 		    
 		    m_buses_out[bus_slot]->init_out();
 			m_buses_out_active[bus_slot] = true;
-
+			m_buses_out_init[bus_slot] = true;
+			
 			if ( !replacement ){
 				m_num_out_buses++;
 			}
 		}	
 		
 		/* the ins */
-		if ( (cap & SND_SEQ_PORT_CAP_SUBS_READ) != 0 &&
-		     snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
-		    
+		if ( (cap & (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ ))
+			 == (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ )
+			 && snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
+	    
 		    m_buses_in[m_num_in_buses] = 
 			new midibus( snd_seq_client_id( m_alsa_seq ),
 				     snd_seq_port_info_get_client(pinfo),
@@ -895,7 +912,8 @@ mastermidibus::port_start( int a_client, int a_port )
 		    
 		    m_buses_in[m_num_in_buses]->init_in();
 		    m_buses_in_active[m_num_in_buses] = true;
-	
+		    m_buses_in_init[m_num_in_buses] = true;
+
 		    m_num_in_buses++;
 		}	
 	}
