@@ -142,6 +142,10 @@ class mastermidibus
 	bool m_buses_out_active[c_maxBuses];
 	bool m_buses_in_active[c_maxBuses];
 
+	bool m_buses_out_init[c_maxBuses];
+	bool m_buses_in_init[c_maxBuses];
+
+
     /* id of queue */
     int m_queue;
 
