@@ -176,9 +176,7 @@ midifile::parse( perform *a_perf, int a_screen_set )
 	    /* we know we have a good track, so we can create
 	       a new sequence to dump it to */
 	    seq = new sequence();
-	    
-	    /* we can set the bus if we get a META event for it,
-	       but for now */
+
 	    seq->set_master_midi_bus( &a_perf->m_master_bus );
 	    
 	    
@@ -230,6 +228,11 @@ midifile::parse( perform *a_perf, int a_screen_set )
 		    
 		    data[0] = m_d[m_pos++];
 		    data[1] = m_d[m_pos++];
+
+			// some files have vel=0 as note off
+			if ( (status & 0xF0) == EVENT_NOTE_ON && data[1] == 0 ){
+				e.set_status( EVENT_NOTE_OFF );
+			}
 		    
 		    //printf( "%02X %02X\n", data[0], data[1] );
 		    
@@ -382,7 +385,7 @@ midifile::parse( perform *a_perf, int a_screen_set )
 
 	    /* the sequence has been filled, add it  */
 	    //printf ( "add_sequence( %d )\n", perf + (a_screen_set * c_seqs_in_set)); 
-	    a_perf->add_sequence( seq, perf + (a_screen_set * c_seqs_in_set    )  );
+	    a_perf->add_sequence( seq, perf + (a_screen_set * c_seqs_in_set)  );
 	}
 	
 	/* dont know what kind of chunk */
@@ -399,80 +402,98 @@ midifile::parse( perform *a_perf, int a_screen_set )
     //printf ( "file_size[%d] m_pos[%d]\n", file_size, m_pos );
 
     if ( (file_size - m_pos) > (int) sizeof( unsigned long)){
-
-	ID = read_long();
-	if ( ID == c_midictrl ){
 		
-		unsigned long seqs = read_long();
- 
-		for ( int i=0; i<seqs; i++ ){
+		ID = read_long();
+		if ( ID == c_midictrl ){
 			
-			a_perf->get_midi_control_toggle(i)->m_active= m_d[m_pos++];
-			a_perf->get_midi_control_toggle(i)->m_inverse_active= m_d[m_pos++];
-			a_perf->get_midi_control_toggle(i)->m_status= m_d[m_pos++];
-			a_perf->get_midi_control_toggle(i)->m_data= m_d[m_pos++];
-			a_perf->get_midi_control_toggle(i)->m_min_value= m_d[m_pos++];
-			a_perf->get_midi_control_toggle(i)->m_max_value= m_d[m_pos++];
+			unsigned long seqs = read_long();
 			
-			a_perf->get_midi_control_on(i)->m_active= m_d[m_pos++];
-			a_perf->get_midi_control_on(i)->m_inverse_active= m_d[m_pos++];
-			a_perf->get_midi_control_on(i)->m_status= m_d[m_pos++];
-			a_perf->get_midi_control_on(i)->m_data= m_d[m_pos++];
-			a_perf->get_midi_control_on(i)->m_min_value= m_d[m_pos++];
-			a_perf->get_midi_control_on(i)->m_max_value= m_d[m_pos++];
-			
-			a_perf->get_midi_control_off(i)->m_active= m_d[m_pos++];
-			a_perf->get_midi_control_off(i)->m_inverse_active= m_d[m_pos++];
-			a_perf->get_midi_control_off(i)->m_status= m_d[m_pos++];
-			a_perf->get_midi_control_off(i)->m_data= m_d[m_pos++];
-			a_perf->get_midi_control_off(i)->m_min_value= m_d[m_pos++];
-			a_perf->get_midi_control_off(i)->m_max_value= m_d[m_pos++];
+			for ( int i=0; i<seqs; i++ ){
+				
+				a_perf->get_midi_control_toggle(i)->m_active= m_d[m_pos++];
+				a_perf->get_midi_control_toggle(i)->m_inverse_active= m_d[m_pos++];
+				a_perf->get_midi_control_toggle(i)->m_status= m_d[m_pos++];
+				a_perf->get_midi_control_toggle(i)->m_data= m_d[m_pos++];
+				a_perf->get_midi_control_toggle(i)->m_min_value= m_d[m_pos++];
+				a_perf->get_midi_control_toggle(i)->m_max_value= m_d[m_pos++];
+				
+				a_perf->get_midi_control_on(i)->m_active= m_d[m_pos++];
+				a_perf->get_midi_control_on(i)->m_inverse_active= m_d[m_pos++];
+				a_perf->get_midi_control_on(i)->m_status= m_d[m_pos++];
+				a_perf->get_midi_control_on(i)->m_data= m_d[m_pos++];
+				a_perf->get_midi_control_on(i)->m_min_value= m_d[m_pos++];
+				a_perf->get_midi_control_on(i)->m_max_value= m_d[m_pos++];
+				
+				a_perf->get_midi_control_off(i)->m_active= m_d[m_pos++];
+				a_perf->get_midi_control_off(i)->m_inverse_active= m_d[m_pos++];
+				a_perf->get_midi_control_off(i)->m_status= m_d[m_pos++];
+				a_perf->get_midi_control_off(i)->m_data= m_d[m_pos++];
+				a_perf->get_midi_control_off(i)->m_min_value= m_d[m_pos++];
+				a_perf->get_midi_control_off(i)->m_max_value= m_d[m_pos++];
+			}
 		}
-	}
-	
-
-	/* Get ID + Length */
-	ID = read_long();
-	if ( ID == c_midiclocks ){
-
-	    TrackLength = read_long();
-	    /* TrackLength is nyumber of buses */
- 	    for ( unsigned int x=0; x<TrackLength; x++ ){
 		
- 		int bus_on =  m_d[m_pos++];
-		a_perf->get_master_midi_bus( )->set_clock( x, (bool) bus_on );
- 	    }
- 	}
+		
+		/* Get ID + Length */
+		ID = read_long();
+		if ( ID == c_midiclocks ){
+			
+			TrackLength = read_long();
+			/* TrackLength is nyumber of buses */
+			for ( unsigned int x=0; x<TrackLength; x++ ){
+				
+				int bus_on =  m_d[m_pos++];
+				a_perf->get_master_midi_bus( )->set_clock( x, (bool) bus_on );
+			}
+		}
+    }
+	
+	if ( (file_size - m_pos) > (int) sizeof( unsigned long)){
+		
+		/* Get ID + Length */
+		ID = read_long();
+		if ( ID == c_notes ){
+			
+			long screen_sets = read_short();
+			
+			for ( unsigned int x=0; x<screen_sets; x++ ){
+				
+				/* get the length of the string */
+				unsigned int len = read_short();
+				
+				char *notes = new char[len + 1];
+				
+				for ( unsigned int i=0; i<len; i++ )
+					notes[i] = m_d[m_pos++];
+				notes[len] = 0;
+				
+				string notess( notes );
+				
+				a_perf->set_screen_set_notepad(x, &notess);
+				
+				delete[] notes;
+			}
+		}
     }
 
-   if ( (file_size - m_pos) > (int) sizeof( unsigned long)){
 
-	/* Get ID + Length */
-	ID = read_long();
-	if ( ID == c_notes ){
-
-	    long screen_sets = read_short();
-	    
-	    for ( unsigned int x=0; x<screen_sets; x++ ){
+	if ( (file_size - m_pos) > (int) sizeof( unsigned long)){
 		
-		/* get the length of the string */
-		unsigned int len = read_short();
-
-		char *notes = new char[len + 1];
-
-		for ( unsigned int i=0; i<len; i++ )
-		    notes[i] = m_d[m_pos++];
-		notes[len] = 0;
-	
-		string notess( notes );
-		
-		a_perf->set_screen_set_notepad(x, &notess);
-
-		delete[] notes;
- 	    }
- 	}
+		/* Get ID + Length */
+		ID = read_long();
+		if ( ID == c_bpmtag ){
+			
+			long bpm = read_long();
+			a_perf->set_bpm(bpm);
+		}
     }
 
+
+
+
+	// *** ADD NEW TAGS AT END **************/
+
+
     delete[] m_d;
     //printf ( "done\n");
 }
@@ -606,6 +627,11 @@ midifile::write( perform *a_perf  )
 	    m_l.push_front( (*note)[j] );
     }
     
+
+	/* bpm */
+    write_long( c_bpmtag );
+    write_long( a_perf->get_bpm() );  
+
     int data_size = m_l.size();
     m_d = (unsigned char *) new char[data_size];
 
