@@ -54,8 +54,34 @@ perform::perform()
     }
 
 	m_offset = 0;
+	m_control_status = 0;
 }
 
+
+void
+perform::clear_all( void )
+{
+
+	reset_sequences();
+
+    for (int i=0; i< c_max_sequence; i++ ){
+
+		if ( m_seqs_active[i] )
+			delete_sequence( i );
+    }
+
+	string e( "" );
+
+	for (int i=0; i<c_max_sets; i++ ){
+		set_screen_set_notepad( i, &e );
+	}
+
+}
+
+
+
+
+
 perform::~perform()
 {
     m_inputing = false;
@@ -120,7 +146,7 @@ perform::add_sequence( sequence *a_seq, int a_perf )
 
     } else {
 
-	for (int i=0; i< c_max_sequence; i++ ){
+	for (int i=a_perf; i< c_max_sequence; i++ ){
 	    
 	    if ( m_seqs_active[i] == false ){
 
@@ -158,56 +184,6 @@ perform::get_master_midi_bus( )
 
 
 
-void
-perform::sequence_queued_toggle( int a_sequence )
-{
-    if ( m_seqs_active[a_sequence] == true ){
-		assert( m_seqs[a_sequence] );
-		m_seqs[a_sequence]->toggle_queued( );
-    } 
-}
-
-
-void
-perform::sequence_queued_off( int a_sequence )
-{
-    if ( m_seqs_active[a_sequence] == true ){
-		assert( m_seqs[a_sequence] );
-		m_seqs[a_sequence]->off_queued( );
-    } 
-}
-
-
-void
-perform::sequence_playing_toggle( int a_sequence )
-{
-    if ( m_seqs_active[a_sequence] == true ){
-	assert( m_seqs[a_sequence] );
-	m_seqs[a_sequence]->toggle_playing();
-    } 
-}
-
-
-void
-perform::sequence_playing_on( int a_sequence )
-{
-    if ( m_seqs_active[a_sequence] == true ){
-		assert( m_seqs[a_sequence] );
-		m_seqs[a_sequence]->set_playing(true);
-    } 
-}
-
-
-void
-perform::sequence_playing_off( int a_sequence )
-{
-    if ( m_seqs_active[a_sequence] == true ){
-		assert( m_seqs[a_sequence] );
-		m_seqs[a_sequence]->set_playing(false);
-    } 
-}
-
-
 
 
 void 
@@ -240,17 +216,24 @@ perform::get_bpm( )
 void
 perform::delete_sequence( int a_num )
 {
-  m_seqs_active[a_num] = false;
-
-    if ( m_seqs[a_num] != NULL ){
-
-	m_seqs[a_num]->set_playing( false );
-	delete m_seqs[a_num];
-
+	m_seqs_active[a_num] = false;
+  
+    if ( m_seqs[a_num] != NULL &&
+         !m_seqs[a_num]->get_editing() ){
+		
+		m_seqs[a_num]->set_playing( false );
+		delete m_seqs[a_num];
     }   
 
 }
 
+bool 
+perform::is_sequence_in_edit( int a_num )
+{
+	return ( m_seqs[a_num] != NULL &&
+			 m_seqs[a_num]->get_editing());
+
+}
 
 void 
 perform::new_sequence( int a_sequence )
@@ -346,8 +329,6 @@ perform::play( long a_tick )
 				
 				m_seqs[i]->play( m_seqs[i]->get_queued_tick() - 1, m_playback_mode );
 				m_seqs[i]->toggle_playing();
-				m_seqs[i]->toggle_queued();
-				
 			}
 			
 			m_seqs[i]->play( a_tick, m_playback_mode );
@@ -370,6 +351,15 @@ perform::set_orig_ticks( long a_tick  )
     }
 }
 
+void 
+perform::clear_sequence_triggers( int a_seq  )
+{
+	if ( m_seqs_active[a_seq] == true ){
+		assert( m_seqs[a_seq] );
+		m_seqs[a_seq]->clear_triggers( );
+	} 
+}
+
 void
 perform::move_triggers( bool a_direction )
 {
@@ -632,7 +622,7 @@ output_thread_func(void *a_pef )
 	clock_tick   += (clock_tick_f / 1000 );
 	clock_tick_f %= 1000;
 
-	if ( p->m_looping ){
+	if ( p->m_looping && p->m_playback_mode ){
 
 	    if ( elapsed_tick >= p->get_right_tick() ){
 		
@@ -807,6 +797,7 @@ output_thread_func(void *a_pef )
 
     p->m_elapsed_tick_copy = elapsed_tick;
     p->m_tick = 0;
+	p->m_master_bus.flush( );
     p->m_master_bus.stop();
     pthread_exit(0);
 }
@@ -916,7 +907,8 @@ input_thread_func(void *a_pef )
 						if( global_showmidi ) 
 							ev.print();
 						
-						p->m_master_bus.sysex( &ev );   
+						if( global_pass_sysex )
+							p->m_master_bus.sysex( &ev );   
 					}
 				}
 				
@@ -928,7 +920,159 @@ input_thread_func(void *a_pef )
 }
 
 
+//	m_mainperf->set_sequence_control_status( c_status_snapshot );
+						/*
+						for( int i=0; i<c_total_seqs; i++ ){
+							
+							if ( m_mainperf->is_active( i ) ){
+								m_sequence_state[i] = m_mainperf->get_sequence( i )->get_playing();
+							}
+							else
+								m_sequence_state[i] = false;
+						}
+						m_l_alt_pressed = true;
+						*/
+
+// sequence key
+		/*
+		if (  m_r_control_pressed ){
+			m_mainperf->sequence_queued_toggle( a_seq + offset );
+		}
+		else {
+			m_mainperf->sequence_playing_toggle( a_seq + offset );
+			m_mainperf->sequence_queued_off( a_seq + offset );
+		}	
+		*/	
+/*
+
+			if ( m_l_control_pressed ){
+						m_mainperf->set_sequence_control_status( c_status_snapshot );
+						m_l_control_pressed = false;
+						m_mainperf->off_sequences( );
+						m_main_wid->reset();
+					}
+*/				
+					//m_main_wid-sequence_key( key_events[a_ev->keyval], m_r_control_pressed );
 
 
 
+/* state resume
+
+	for( int i=0; i<c_total_seqs; i++ ){
+				
+				if ( m_mainperf->is_active( i ) ){
+					m_mainperf->get_sequence( i )->set_playing( m_sequence_state[i] );
+				}
+			}
+			
+			m_main_wid->reset();
+			m_l_alt_pressed = false;
+
+*/
+
+void 
+perform::save_playing_state( void )
+{
+	for( int i=0; i<c_total_seqs; i++ ){
+		
+		if ( m_seqs_active[i] == true ){
+			assert( m_seqs[i] );
+			m_sequence_state[i] = m_seqs[i]->get_playing();
+		}
+		else
+			m_sequence_state[i] = false;
+	} 
+}
+
+void 
+perform::restore_playing_state( void )
+{
+	for( int i=0; i<c_total_seqs; i++ ){
+		
+		if ( m_seqs_active[i] == true ){
+			assert( m_seqs[i] );
+			m_seqs[i]->set_playing( m_sequence_state[i] );
+		}
+	} 
+}
+
+
+void 
+perform::set_sequence_control_status( int a_status )
+{
+	if ( a_status & c_status_snapshot ){
+		save_playing_state(  );
+	}
+
+	m_control_status |= a_status;
+}
+
+
+void 
+perform::unset_sequence_control_status( int a_status )
+{
+	if ( a_status & c_status_snapshot ){
+		restore_playing_state(  );
+	}
+
+	m_control_status &= (~a_status);
+}
+
+
+
+void
+perform::sequence_playing_toggle( int a_sequence )
+{
+    if ( m_seqs_active[a_sequence] == true ){
+		assert( m_seqs[a_sequence] );
+
+		if ( m_control_status & c_status_queue ){
+			m_seqs[a_sequence]->toggle_queued();
+		}
+		else {
+
+			if (  m_control_status & c_status_replace ){
+				unset_sequence_control_status( c_status_replace );	
+				off_sequences( );
+			}
+			
+			m_seqs[a_sequence]->toggle_playing();
+
+		}
+    } 
+}
+
+
+void
+perform::sequence_playing_on( int a_sequence )
+{
+    if ( m_seqs_active[a_sequence] == true ){
+		assert( m_seqs[a_sequence] );
+		m_seqs[a_sequence]->set_playing(true);
+    } 
+}
+
+
+void
+perform::sequence_playing_off( int a_sequence )
+{
+    if ( m_seqs_active[a_sequence] == true ){
+		assert( m_seqs[a_sequence] );
+		m_seqs[a_sequence]->set_playing(false);
+    } 
+}
+
+
+
+/*
+void
+perform::sequence_queued_off( int a_sequence )
+{
+    if ( m_seqs_active[a_sequence] == true ){
+		assert( m_seqs[a_sequence] );
+		m_seqs[a_sequence]->off_queued( );
+    } 
+}
+*/
+
 
