@@ -33,6 +33,7 @@ option long_options[] = {
     {"stats",     0, 0, 'S' },
     {"priority", 0, 0, 'p' },
     {"ignore",required_argument, 0, 'i'},
+	{"pass_sysex", 0, 0, 'P'},
     {0, 0, 0, 0}
 
 };
@@ -42,6 +43,7 @@ bool global_priority = false;
 bool global_device_ignore = false;
 int global_device_ignore_num = 0;
 bool global_stats = false;
+bool global_pass_sysex = false;
 
 int 
 main (int argc, char *argv[])
@@ -82,6 +84,8 @@ main (int argc, char *argv[])
 	  printf( "    --file <filename> : load midi file on startup\n" );
 	  printf( "    --showmidi : dumps incoming midi to screen\n" );
   	  printf( "    --priority : runs higher priority with FIFO scheduler (must be root)\n" );
+  	  printf( "    --pass_sysex : passes any incoming sysex messages to all outputs \n" );
+	  printf( "\n\n\n" );
 
 	  return 0;
 	  break;
@@ -103,6 +107,11 @@ main (int argc, char *argv[])
 
 	  break;
 
+	  case 'P':
+      
+          global_pass_sysex = true;
+
+		  break;
 
       case 'f':
     
