@@ -36,9 +36,11 @@
 #include "scale.xpm"
 #include "key.xpm"   
 #include "down.xpm"
+#include "note_length.xpm"
 
 int seqedit::m_initial_zoom = 4;
 int seqedit::m_initial_snap = c_ppqn / 4;
+int seqedit::m_initial_note_length = c_ppqn / 4;
 int seqedit::m_initial_scale = 0;
 int seqedit::m_initial_key = 0;
 
@@ -58,13 +60,15 @@ seqedit::seqedit( sequence *a_seq,
 
 : Gtk::Window( GTK_WINDOW_TOPLEVEL )
 {
+
     /* set the performance */
     m_seq = a_seq;
 
-    m_zoom =  m_initial_zoom;
-    m_snap =  m_initial_snap;
-    m_scale = m_initial_scale;
-    m_key =   m_initial_key;
+    m_zoom        =  m_initial_zoom;
+    m_snap        =  m_initial_snap;
+	m_note_length = m_initial_note_length;
+    m_scale       = m_initial_scale;
+    m_key         =   m_initial_key;
 
     m_mainperf = a_perf;
     m_mainwid = a_mainwid;
@@ -86,9 +90,10 @@ seqedit::seqedit( sequence *a_seq,
     
     /* menus */
     m_menubar   =  manage( new MenuBar());
-    m_menu_action = manage( new Menu());
+	m_menu_tools = manage( new Menu() );
     m_menu_zoom =  manage( new Menu());
     m_menu_snap =   manage( new Menu());
+	m_menu_note_length = manage( new Menu());
     m_menu_length = manage( new Menu());
     m_menu_bpm = manage( new Menu() );
     m_menu_bw = manage( new Menu() );
@@ -180,6 +185,11 @@ seqedit::seqedit( sequence *a_seq,
     /* exapand, cause rollview expands */
     m_vbox->pack_start(*m_table, true, true, 0);
 
+	/* tools button */
+    m_button_tools = manage( new Button( " T " ));
+    m_button_tools->clicked.connect( bind<Menu *>( slot( this, &seqedit::popup_menu), m_menu_tools ));
+
+
 
     /* data button */
     m_button_data = manage( new Button( " Event " ));
@@ -231,6 +241,7 @@ seqedit::seqedit( sequence *a_seq,
 
     set_zoom( m_zoom );
     set_snap( m_snap );
+	set_note_length( m_note_length );
 
 
     set_bpm( m_seq->get_bpm() );
@@ -279,6 +290,24 @@ seqedit::create_menus( void )
     m_menu_snap->items().push_back(MenuElem("1/48",  bind(slot(this,&seqedit::set_snap), c_ppqn / 4  / 3 )));
     m_menu_snap->items().push_back(MenuElem("1/96",  bind(slot(this,&seqedit::set_snap), c_ppqn / 8  / 3 )));
 	m_menu_snap->items().push_back(MenuElem("1/192", bind(slot(this,&seqedit::set_snap), c_ppqn / 16 / 3 )));
+
+    /* note note_length */
+    m_menu_note_length->items().push_back(MenuElem("1",     bind(slot(this,&seqedit::set_note_length), c_ppqn * 4  )));
+    m_menu_note_length->items().push_back(MenuElem("1/2",   bind(slot(this,&seqedit::set_note_length), c_ppqn * 2  )));
+    m_menu_note_length->items().push_back(MenuElem("1/4",   bind(slot(this,&seqedit::set_note_length), c_ppqn * 1  )));
+    m_menu_note_length->items().push_back(MenuElem("1/8",   bind(slot(this,&seqedit::set_note_length), c_ppqn / 2  )));
+    m_menu_note_length->items().push_back(MenuElem("1/16",  bind(slot(this,&seqedit::set_note_length), c_ppqn / 4  )));
+    m_menu_note_length->items().push_back(MenuElem("1/32",  bind(slot(this,&seqedit::set_note_length), c_ppqn / 8  )));
+    m_menu_note_length->items().push_back(MenuElem("1/64",  bind(slot(this,&seqedit::set_note_length), c_ppqn / 16 )));
+    m_menu_note_length->items().push_back(MenuElem("1/128", bind(slot(this,&seqedit::set_note_length), c_ppqn / 32 )));
+	m_menu_note_length->items().push_back(SeparatorElem());
+    m_menu_note_length->items().push_back(MenuElem("1/3",   bind(slot(this,&seqedit::set_note_length), c_ppqn * 4  / 3 )));
+    m_menu_note_length->items().push_back(MenuElem("1/6",   bind(slot(this,&seqedit::set_note_length), c_ppqn * 2  / 3 )));
+    m_menu_note_length->items().push_back(MenuElem("1/12",  bind(slot(this,&seqedit::set_note_length), c_ppqn * 1  / 3 )));
+    m_menu_note_length->items().push_back(MenuElem("1/24",  bind(slot(this,&seqedit::set_note_length), c_ppqn / 2  / 3 )));
+    m_menu_note_length->items().push_back(MenuElem("1/48",  bind(slot(this,&seqedit::set_note_length), c_ppqn / 4  / 3 )));
+    m_menu_note_length->items().push_back(MenuElem("1/96",  bind(slot(this,&seqedit::set_note_length), c_ppqn / 8  / 3 )));
+	m_menu_note_length->items().push_back(MenuElem("1/192", bind(slot(this,&seqedit::set_note_length), c_ppqn / 16 / 3 )));
 	
     /* Key */
     m_menu_key->items().push_back(MenuElem( c_key_text[0],  bind(slot(this,&seqedit::set_key), 0 )));
@@ -358,8 +387,42 @@ seqedit::create_menus( void )
 	m_menu_midibus->items().push_back(MenuElem(masterbus->get_midi_bus_name(i),
 						  bind(slot(this,&seqedit::set_midi_bus), i)));
     }
+
+
+	/* tools */
+	Menu *holder;
+
+	holder = manage( new Menu());
+	holder->items().push_back( MenuElem( "All Notes",       bind(slot(this,&seqedit::do_action),  0 )));
+	holder->items().push_back( MenuElem( "All Events",      bind(slot(this,&seqedit::do_action),  0 )));
+	holder->items().push_back( MenuElem( "Inverse Notes",   bind(slot(this,&seqedit::do_action),  0 )));
+  	holder->items().push_back( MenuElem( "Inverse Events",  bind(slot(this,&seqedit::do_action), 0 )));
+	m_menu_tools->items().push_back( MenuElem( "Select", *holder ));
+
+	holder = manage( new Menu());
+	holder->items().push_back( MenuElem( "Notes",       bind(slot(this,&seqedit::do_action), 0 )));
+  	holder->items().push_back( MenuElem( "Events",      bind(slot(this,&seqedit::do_action), 0 )));
+ 	holder->items().push_back( MenuElem( "All",         bind(slot(this,&seqedit::do_action), 0 )));
+ 	holder->items().push_back( MenuElem( "Selected",    bind(slot(this,&seqedit::do_action), 0 )));
+	m_menu_tools->items().push_back( MenuElem( "Quantize", *holder ));
+ 
+	holder = manage( new Menu());
+	holder->items().push_back( MenuElem( "Up",       bind(slot(this,&seqedit::do_action), 0 )));
+  	holder->items().push_back( MenuElem( "Down",      bind(slot(this,&seqedit::do_action), 0 )));
+
+	m_menu_tools->items().push_back( MenuElem( "Transpose", *holder ));
+
+
+
+  //m_menu_tools->items().push_back( SeparatorElem( )); 
+ 
 }
 
+void
+seqedit::do_action( int a_action )
+{
+
+}
 
 
 void  
@@ -373,6 +436,7 @@ seqedit::fill_top_bar( void )
     m_entry_name->changed.connect( slot( this, &seqedit::name_change_callback));
 
     m_hbox->pack_start( *m_entry_name, true, true );
+	//m_hbox->pack_start( *m_button_tools, false, false );
 
     m_hbox->pack_start( *(manage(new VSeparator( ))), false, false, 4);
 
@@ -456,6 +520,20 @@ seqedit::fill_top_bar( void )
     m_hbox2->pack_start( *m_button_snap , false, false );
     m_hbox2->pack_start( *m_entry_snap , false, false );
 
+
+    /* note_length */
+    m_button_note_length = manage( new Button());
+    m_button_note_length->add( *manage( new Pixmap( note_length_xpm  )));
+    m_button_note_length->clicked.connect(  bind<Menu *>( slot( this, &seqedit::popup_menu), m_menu_note_length  ));
+    m_tooltips->set_tip( *m_button_note_length, "Note Length." );
+    m_entry_note_length = manage( new Entry());
+    m_entry_note_length->set_usize( 36, -1 );
+    m_entry_note_length->set_editable( false );
+    
+    m_hbox2->pack_start( *m_button_note_length , false, false );
+    m_hbox2->pack_start( *m_entry_note_length , false, false );
+
+
     /* zoom */
     m_button_zoom = manage( new Button());
     m_button_zoom->add( *manage( new Pixmap( zoom_xpm  )));
@@ -523,7 +601,8 @@ void
 seqedit::set_midi_bus( int a_midibus )
 {
     m_seq->set_midi_bus( a_midibus );
-    m_entry_bus->set_text( m_mainperf->get_master_midi_bus()->get_midi_bus_name( a_midibus ));
+	mastermidibus *mmb =  m_mainperf->get_master_midi_bus();
+    m_entry_bus->set_text( mmb->get_midi_bus_name( a_midibus ));
     m_mainwid->update_sequence_on_window( m_pos );
 }
 
@@ -559,6 +638,19 @@ seqedit::set_snap( int a_snap  )
 
 
 
+void 
+seqedit::set_note_length( int a_note_length  )
+{
+    char b[10];
+    sprintf( b, "1/%d",   c_ppqn * 4 / a_note_length );
+    m_entry_note_length->set_text(b);
+    
+    m_note_length = a_note_length;
+    m_initial_note_length = a_note_length;
+    m_seqroll_wid->set_note_length( m_note_length );
+}
+
+
 
 
 void 
@@ -694,6 +786,7 @@ seqedit::set_data_type( unsigned char a_status, unsigned char a_control  )
 
     m_seqevent_wid->set_data_type( a_status, a_control );
     m_seqdata_wid->set_data_type( a_status, a_control );
+	m_seqroll_wid->set_data_type( a_status, a_control );
 
     char text[100];
     char hex[20];
@@ -737,7 +830,7 @@ int
 seqedit::timeout( void )
 {
 
-    if (m_seq->is_dirty() ){
+    if (m_seq->is_dirty_edit() ){
 
 	m_seqroll_wid->idle_redraw();
 	m_seqevent_wid->idle_redraw();
