@@ -64,6 +64,7 @@ seqroll::seqroll(sequence *a_seq,
 
     m_selecting = false;
     m_moving    = false;
+	m_moving_init = false;
     m_growing   = false;
     m_adding    = false;
     m_paste     = false;
@@ -248,6 +249,13 @@ seqroll::set_snap( int a_snap )
     m_snap = a_snap;
 }
 
+
+void 
+seqroll::set_note_length( int a_note_length )
+{
+	m_note_length = a_note_length;
+}
+
 /* sets the music scale */
 void 
 seqroll::set_scale( int a_scale )
@@ -670,9 +678,7 @@ seqroll::button_press_event_impl(GdkEventButton* a_ev)
 		m_paste = false;
 		m_seq->paste_selected( tick_s, note_h );
     }
-	
 
-	
     /*      left mouse button     */
     if ( a_ev->button == 1 ){ 
 
@@ -701,14 +707,14 @@ seqroll::button_press_event_impl(GdkEventButton* a_ev)
 			convert_xy( m_drop_x, m_drop_y, &tick_s, &note_h );
 			
 			/* add note, length = little less than snap */
-			m_seq->add_note( tick_s, m_snap - 2, note_h );
+			m_seq->add_note( tick_s, m_note_length - 2, note_h );
 		}
 		
 		/* if we clicked on an event, we can start moving all selected
 		   notes */
 		else if ( numsel > 0 ){
 			
-			m_moving = true;
+			m_moving_init = true;
 			
 			/* moving, snapped x */
 			m_current_x = m_drop_x = snapped_x;
@@ -814,7 +820,7 @@ seqroll::button_release_event_impl(GdkEventButton* a_ev)
 	    numsel = m_seq->select_note_events( tick_s, note_h, tick_f, note_l +1 );
 	}
 
-	if ( m_moving ){
+	if (  m_moving  ){
 	    
 	    /* convert deltas into screen corridinates */
 	    convert_xy( delta_x, delta_y, &delta_tick, &delta_note );
@@ -846,6 +852,7 @@ seqroll::button_release_event_impl(GdkEventButton* a_ev)
     m_moving = false;
     m_growing = false;
     m_paste = false;
+	m_moving_init = false;
 
     /* if they clicked, something changed */
 
@@ -869,20 +876,25 @@ seqroll::motion_notify_event_impl(GdkEventMotion* a_ev)
     int note;
     long tick;
 
+	if ( m_moving_init ){
+		m_moving_init = false;
+		m_moving = true;
+	}
+		 
     
     snap_y( &m_current_y );
     convert_xy( 0, m_current_y, &tick, &note );
-
+	
     m_seqkeys_wid->set_hint_key( note );
-
+	
     if ( m_selecting || m_moving || m_growing || m_paste ){
-
-	if ( m_moving || m_paste )
-	    snap_x( &m_current_x );
- 
-	draw_selection_on_window();
-	return true;
-
+		
+		if ( m_moving || m_paste )
+			snap_x( &m_current_x );
+		
+		draw_selection_on_window();
+		return true;
+		
     }
     return false;
 }
@@ -1002,3 +1014,12 @@ seqroll::key_press_event_impl(GdkEventKey* a_p0)
 	return false;
 }
 
+
+
+
+void 
+seqroll::set_data_type( unsigned char a_status, unsigned char a_control = 0 )
+{
+    m_status = a_status;
+    m_cc = a_control;
+}
