@@ -83,15 +83,21 @@ class seqroll : public Gtk::DrawingArea
     /* one pixel == m_zoom ticks */
     int          m_zoom;
     int          m_snap;
+    int          m_note_length;
 
     int          m_scale;
     int          m_key;
 
     int m_window_x, m_window_y;
 
+	/* what is the data window currently editing ? */
+    unsigned char m_status;
+    unsigned char m_cc;
+
     /* when highlighting a bunch of events */
     bool m_selecting;
     bool m_moving;
+	bool m_moving_init;
     bool m_growing;
     bool m_adding;
     bool m_paste;
@@ -135,6 +141,7 @@ class seqroll : public Gtk::DrawingArea
 	
     void draw_events_on( Gdk_Drawable *a_draw );
 
+  
     int idle_progress();
 
  public:
@@ -142,6 +149,7 @@ class seqroll : public Gtk::DrawingArea
     void reset();
     void set_zoom( int a_zoom );
     void set_snap( int a_snap );
+	void set_note_length( int a_note_length );
     
     void set_scale( int a_scale );
     void set_key( int a_key );
@@ -166,6 +174,8 @@ class seqroll : public Gtk::DrawingArea
 	     mainwid *a_mainwid, 
 	     int a_pos );
 
+	void set_data_type( unsigned char a_status, unsigned char a_control  );
+ 
     ~seqroll( );
 };
 
