@@ -32,7 +32,6 @@ sequence::sequence( )
     m_was_playing   = false;
     m_recording     = false;
     m_thru          = false;
-    m_dirty         = false;
 	m_queued        = false;
 
     m_time_beats_per_measure = 4;
@@ -47,18 +46,25 @@ sequence::sequence( )
   
     /* no notes are playing */
     for (int i=0; i< c_midi_notes; i++ )
-	m_playing_notes[i] = false;
+		m_playing_notes[i] = 0;
 
     m_last_tick = 0;
 
 	m_masterbus = NULL;
+	m_dirty_main = true;
+	m_dirty_edit = true;
 }
 
 
+
 void 
 sequence::set_master_midi_bus( mastermidibus *a_mmb )
 {
+	lock();
+
 	m_masterbus = a_mmb;
+
+	unlock();
 }
 
 
@@ -66,7 +72,9 @@ sequence::set_master_midi_bus( mastermidibus *a_mmb )
 void 
 sequence::set_bpm( long a_beats_per_measure )
 {
-  m_time_beats_per_measure = a_beats_per_measure;
+	lock();
+	m_time_beats_per_measure = a_beats_per_measure;
+	unlock();
 }
 
 long 
@@ -78,7 +86,10 @@ sequence::get_bpm( void )
 void 
 sequence::set_bw( long a_beat_width )
 {
-  m_time_beat_width = a_beat_width;
+	lock();
+	m_time_beat_width = a_beat_width;
+	m_dirty_main = m_dirty_edit = true;
+	unlock();
 }
 
 long 
@@ -111,7 +122,9 @@ sequence::add_event( const event *a_e )
 void 
 sequence::set_orig_tick( long a_tick )
 {
+	lock();
     m_last_tick = a_tick;
+	unlock();
 }
 
 
@@ -120,6 +133,8 @@ sequence::toggle_queued( void )
 {
 	lock();
 
+	m_dirty_main = m_dirty_edit = true;
+
 	m_queued = !m_queued;
 	m_queued_tick = m_last_tick - (m_last_tick % m_length) + m_length;
 
@@ -132,6 +147,8 @@ sequence::off_queued( void )
 
 	lock();
 
+	m_dirty_main = m_dirty_edit = true;
+
 	m_queued = false;
 
 	unlock();
@@ -583,9 +600,10 @@ sequence::remove_selected( )
 	    /* if its a note off, and that note is currently
 	       playing, send a note off */
 	    if ( (*i).is_note_off()  &&
-		 m_playing_notes[ (*i).get_note()] ){
+		 m_playing_notes[ (*i).get_note()] > 0 ){
 		
-		m_masterbus->play( m_bus, &(*i), m_midi_channel ); 
+			m_masterbus->play( m_bus, &(*i), m_midi_channel );
+			m_playing_notes[(*i).get_note()]--;
 	    }
 	   
 	    t = i; t++;
@@ -680,6 +698,62 @@ sequence::get_clipboard_box( long *a_tick_s, int *a_note_h,
 }
 
 
+
+int 
+sequence::get_num_selected_notes( )
+{
+	int ret = 0;
+
+	list<event>::iterator i;
+
+    lock();
+
+    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+
+        if( (*i).is_note_on()                &&
+			(*i).is_selected() ){
+			ret++;
+		}
+    }
+
+    unlock();
+
+	return ret;
+
+}
+
+
+int 
+sequence::get_num_selected_events( unsigned char a_status, 
+								   unsigned char a_cc )
+{
+	int ret = 0;
+	list<event>::iterator i;
+
+	lock();
+
+    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+
+		if( (*i).get_status()    == a_status ){
+
+			unsigned char d0,d1;
+			(*i).get_data( &d0, &d1 );
+
+			if ( (a_status == EVENT_CONTROL_CHANGE && d0 == a_cc )
+				 || (a_status != EVENT_CONTROL_CHANGE) ){
+
+				if ( (*i).is_selected( ))
+					ret++;
+			}
+	    }
+	}
+    
+	unlock();
+    
+    return ret;
+}
+
+
 /* selects events in range..  tick start, note high, tick end
    note low */
 int
@@ -837,6 +911,8 @@ sequence::move_selected_notes( long a_delta_tick, int a_delta_note )
 }
 
 
+
+
 /* moves note off event */
 void 
 sequence::grow_selected( long a_delta_tick )
@@ -883,6 +959,74 @@ sequence::grow_selected( long a_delta_tick )
 }
 
 
+void 
+sequence::increment_selected( unsigned char a_status, unsigned char a_control )
+{
+    lock();
+
+    list<event>::iterator i;
+
+    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+	
+		if ( (*i).is_selected() &&
+			 (*i).get_status() == a_status ){
+			
+			if ( a_status == EVENT_NOTE_ON || 
+				 a_status == EVENT_NOTE_OFF || 
+				 a_status == EVENT_AFTERTOUCH ||
+				 a_status == EVENT_CONTROL_CHANGE || 
+				 a_status == EVENT_PITCH_WHEEL ){
+				
+				(*i).increment_data2();
+			}
+			
+			if ( a_status == EVENT_PROGRAM_CHANGE ||
+				 a_status == EVENT_CHANNEL_PRESSURE ){
+				
+				(*i).increment_data1();
+			}
+		}
+    }
+        
+    unlock();
+}
+
+
+void 
+sequence::decrement_selected(unsigned char a_status, unsigned char a_control )
+{
+    lock();
+
+    list<event>::iterator i;
+
+    for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
+	
+	   	if ( (*i).is_selected() &&
+			 (*i).get_status() == a_status ){
+			
+			if ( a_status == EVENT_NOTE_ON || 
+				 a_status == EVENT_NOTE_OFF || 
+				 a_status == EVENT_AFTERTOUCH ||
+				 a_status == EVENT_CONTROL_CHANGE || 
+				 a_status == EVENT_PITCH_WHEEL ){
+				
+				(*i).decrement_data2();
+			}
+			
+			if ( a_status == EVENT_PROGRAM_CHANGE ||
+				 a_status == EVENT_CHANNEL_PRESSURE ){
+				
+				(*i).decrement_data1();
+			}
+			
+		}
+	}
+        
+	unlock();
+}
+
+
+
 void sequence::clear_clipboard( void )
 {
     m_list_clipboard->clear( );
@@ -893,6 +1037,8 @@ sequence::copy_selected( void )
 {
     list<event>::iterator i;
 
+	lock();
+
     m_list_clipboard->clear( );
 
     for ( i = m_list_event->begin(); i != m_list_event->end(); i++ ){
@@ -908,15 +1054,21 @@ sequence::copy_selected( void )
 
 	(*i).set_timestamp((*i).get_timestamp() - first_tick );
     }
+
+	unlock();
 }
 
 void 
 sequence::paste_selected( long a_tick, int a_note )
 {
 
+
+
     list<event>::iterator i;
     int highest_note = 0;
 
+	lock();
+
     for ( i = m_list_clipboard->begin(); i != m_list_clipboard->end(); i++ ){
 	(*i).set_timestamp((*i).get_timestamp() + a_tick );
     }
@@ -942,6 +1094,8 @@ sequence::paste_selected( long a_tick, int a_note )
 
     reset_draw_marker();
     
+	unlock();
+
 }
 
 
@@ -1094,7 +1248,7 @@ sequence::stream_event(  event *a_ev  )
     if ( m_recording ){
 	
 	add_event( a_ev );
-	m_dirty = true;
+	m_dirty_main = m_dirty_edit = true;
     }
 
     if ( m_thru ){
@@ -1109,12 +1263,26 @@ sequence::stream_event(  event *a_ev  )
 } 
 
 bool
-sequence::is_dirty( )
+sequence::is_dirty_main( )
 {
     lock();
 
-    bool ret = m_dirty;
-    m_dirty = false;
+    bool ret = m_dirty_main;
+    m_dirty_main = false;
+
+    unlock();
+
+    return ret;
+}
+
+
+bool
+sequence::is_dirty_edit( )
+{
+    lock();
+
+    bool ret = m_dirty_edit;
+    m_dirty_edit = false;
 
     unlock();
 
@@ -1158,6 +1326,15 @@ sequence::play_note_off( int a_note )
 }
 
 
+void 
+sequence::clear_triggers( void )
+{
+	lock();
+	m_list_trigger->clear();
+	unlock();
+
+}
+
 
 /* adds trigger, a_state = true, range is on.
                  a_state = false, range is off */ 
@@ -1267,6 +1444,8 @@ sequence::validate_triggers( void  )
     list<trigger>::iterator i = m_list_trigger->begin();
     list<trigger>::iterator j = m_list_trigger->begin();
 
+	lock();
+
     j++;
     
     while(  j != m_list_trigger->end() ){
@@ -1284,6 +1463,8 @@ sequence::validate_triggers( void  )
  	    i++; j++;
  	}
     }
+
+	unlock();
 }
 
 void 
@@ -1292,6 +1473,8 @@ sequence::move_triggers( long a_start_tick,
 			 bool a_direction )
 {
 
+	lock();
+
     long first_tick = a_start_tick;
 
     if ( ! a_direction ){
@@ -1319,6 +1502,8 @@ sequence::move_triggers( long a_start_tick,
     if ( a_direction ){
 	add_trigger( a_start_tick, a_distance, false );
     }
+
+	unlock();
 }
 
 
@@ -1329,6 +1514,8 @@ sequence::copy_triggers( long a_start_tick,
 
     long first_tick = a_start_tick;
 
+	lock();
+
     move_triggers( a_start_tick, 
 		   a_distance, 
 		   true );
@@ -1366,7 +1553,7 @@ sequence::copy_triggers( long a_start_tick,
     m_list_trigger->sort();
     validate_triggers( );
 
-
+	unlock();
 }
 
 
@@ -1416,13 +1603,21 @@ sequence::get_trigger_state( long a_tick )
 void 
 sequence::reset_draw_marker( void )
 {
+	lock();
+
     m_iterator_draw = m_list_event->begin();
+
+	unlock();
 }
 
 void
 sequence::reset_draw_trigger_marker( void )
 {
+	lock();
+
     m_iterator_draw_trigger = m_list_trigger->begin();
+
+	unlock();
 }
 
 
@@ -1605,7 +1800,7 @@ sequence::operator= (const sequence& a_rhs)
 
 	/* no notes are playing */
 	for (int i=0; i< c_midi_notes; i++ )
-	    m_playing_notes[i] = false;
+	    m_playing_notes[i] = 0;
 
 	/* reset */
 	zero_markers( );
@@ -1658,6 +1853,7 @@ sequence::set_midi_bus( char  a_mb )
 	off_playing_notes( );
 
     this->m_bus = a_mb;
+	m_dirty_main = m_dirty_edit = true;
 
     unlock();
 }
@@ -1721,7 +1917,10 @@ sequence::set_playing( bool a_p )
 	m_playing = false;
 	off_playing_notes();
 
-    }
+    }  
+
+	m_dirty_main = m_dirty_edit = true;
+	m_queued = false;
 
     unlock();
 }
@@ -1793,6 +1992,7 @@ sequence::set_midi_channel( unsigned char a_ch )
     lock();
     off_playing_notes( );
     m_midi_channel = a_ch;
+	m_dirty_main = m_dirty_edit = true;
     unlock();
 }
 
@@ -1822,17 +2022,26 @@ sequence::put_event_on_bus( event *a_e )
     lock();
 
     char note = a_e->get_note();
-
- 
+	bool skip = false;
+	
     if ( a_e->is_note_on() ){
-
-	m_playing_notes[note] = true;
+		
+		m_playing_notes[note]++;
     }
     if ( a_e->is_note_off() ){
- 
-	m_playing_notes[note] = false;		
+
+		if (  m_playing_notes[note] <= 0 ){
+			skip = true;
+		}
+		else {
+			m_playing_notes[note]--;
+		}		
     }
-    m_masterbus->play( m_bus, a_e,  m_midi_channel );   
+
+	if ( !skip ){
+		m_masterbus->play( m_bus, a_e,  m_midi_channel );   
+	}
+
     m_masterbus->flush();
     
     unlock();
@@ -1849,15 +2058,15 @@ sequence::off_playing_notes()
     
     for ( int x=0; x< c_midi_notes; x++ ){
 	
-	if( m_playing_notes[x] ){
-	    
-	    e.set_status( EVENT_NOTE_OFF );
-	    e.set_data( x, 0 );
-		
-	    m_masterbus->play( m_bus, &e, m_midi_channel ); 
-	    
-	    m_playing_notes[x] = false;
-	}
+		while( m_playing_notes[x] > 0 ){
+			
+			e.set_status( EVENT_NOTE_OFF );
+			e.set_data( x, 0 );
+			
+			m_masterbus->play( m_bus, &e, m_midi_channel ); 
+			
+			m_playing_notes[x]--;
+		}
     }
     
     m_masterbus->flush();
